"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseJsonLogin = void 0;
function parseJsonLogin(jsonString) {
    let json = {};
    try {
        json = JSON.parse(jsonString.trim());
    }
    catch (e) {
        throw new Error('json parse failed');
    }
    if (typeof json !== 'object') {
        throw new Error(`json is not valid (typeof = ${typeof json})`);
    }
    let { accountId, password, accountType, type, rateLimit, host, appName, tradingDisabled } = json;
    accountType || (accountType = type);
    if (typeof accountId !== 'string' ||
        typeof password !== 'string' ||
        typeof accountType !== 'string' ||
        !['undefined', 'number'].includes(typeof rateLimit) ||
        !['undefined', 'string'].includes(typeof host) ||
        !['undefined', 'string'].includes(typeof appName) ||
        !['undefined', 'boolean'].includes(typeof tradingDisabled) ||
        Object.keys(json).length > 7) {
        throw new Error('json is not valid');
    }
    if (!accountType || ['real', 'demo'].every(x => x !== (accountType === null || accountType === void 0 ? void 0 : accountType.toLowerCase()))) {
        throw new Error('json not contains valid "accountType" (it should be "real" or "demo")');
    }
    return {
        accountId,
        password,
        accountType: accountType.toLowerCase(),
        rateLimit,
        host,
        appName,
        tradingDisabled
    };
}
exports.parseJsonLogin = parseJsonLogin;
//# sourceMappingURL=parseJsonLogin.js.map